@@ -1,8 +1,11 @@
 import os
-import sys
-import pytest
 import shutil
+import sys
+
 import erfa  # noqa
+import matplotlib
+import pytest
+
 import astropy  # noqa
 
 if len(sys.argv) == 3 and sys.argv[1] == '--astropy-root':
@@ -62,6 +65,10 @@ os.remove(os.path.join('astropy_tests', 'modeling', 'tests', 'test_convolution.p
 os.remove(os.path.join('astropy_tests', 'modeling', 'tests', 'test_core.py'))
 os.remove(os.path.join('astropy_tests', 'visualization', 'tests', 'test_lupton_rgb.py'))
 
+# FIXME: PIL minversion check does not work
+os.remove(os.path.join('astropy_tests', 'visualization', 'wcsaxes', 'tests', 'test_misc.py'))
+os.remove(os.path.join('astropy_tests', 'visualization', 'wcsaxes', 'tests', 'test_wcsapi.py'))
+
 # FIXME: The following tests rely on the fully qualified name of classes which
 # don't seem to be the same.
 os.remove(os.path.join('astropy_tests', 'table', 'mixins', 'tests', 'test_registry.py'))
@@ -70,6 +77,10 @@ os.remove(os.path.join('astropy_tests', 'table', 'mixins', 'tests', 'test_regist
 shutil.copy2(os.path.join(ROOT, 'astropy', 'conftest.py'),
              os.path.join('astropy_tests', 'conftest.py'))
 
+# matplotlib hook in pyinstaller 5.0 and later no longer collects every backend, see
+# https://github.com/pyinstaller/pyinstaller/issues/6760
+matplotlib.use('svg')
+
 # We skip a few tests, which are generally ones that rely on explicitly
 # checking the name of the current module (which ends up starting with
 # astropy_tests rather than astropy).
@@ -95,7 +106,8 @@ SKIP_TESTS = ['test_exception_logging_origin',
 # Run the tests!
 sys.exit(pytest.main(['astropy_tests',
                       '-k ' + ' and '.join('not ' + test for test in SKIP_TESTS)],
-                     plugins=['pytest_doctestplus.plugin',
+                     plugins=['pytest_astropy.plugin',
+                              'pytest_doctestplus.plugin',
                               'pytest_openfiles.plugin',
                               'pytest_remotedata.plugin',
                               'pytest_astropy_header.display']))
