@@ -24,10 +24,13 @@ class Conf(_config.ConfigNamespace):
     Configuration parameters for `astropy.io.votable`.
     """
 
-    pedantic = _config.ConfigItem(
-        False,
-        'When True, treat fixable violations of the VOTable spec as exceptions.',
-        aliases=['astropy.io.votable.table.pedantic'])
+    verify = _config.ConfigItem(
+        'ignore',
+        "Can be 'exception' (treat fixable violations of the VOTable spec as "
+        "exceptions), 'warn' (show warnings for VOTable spec violations), or "
+        "'ignore' (silently ignore VOTable spec violations)",
+        aliases=['astropy.io.votable.table.pedantic',
+                 'astropy.io.votable.pedantic'])
 
 
 conf = Conf()
