@@ -4,29 +4,29 @@ Module to test fitting routines
 """
 # pylint: disable=invalid-name
 import os.path
-import warnings
-from unittest import mock
-from importlib.metadata import EntryPoint
-
-import pytest
-import numpy as np
 import unittest.mock as mk
+from importlib.metadata import EntryPoint
+from itertools import combinations
+from unittest import mock
+
+import numpy as np
+import pytest
 from numpy import linalg
 from numpy.testing import assert_allclose, assert_almost_equal, assert_equal
 
 from astropy.modeling import models
 from astropy.modeling.core import Fittable2DModel, Parameter
 from astropy.modeling.fitting import (
-    SimplexLSQFitter, SLSQPLSQFitter, LinearLSQFitter, LevMarLSQFitter,
-    JointFitter, Fitter, FittingWithOutlierRemoval)
+    DogBoxLSQFitter, Fitter, FittingWithOutlierRemoval, JointFitter, LevMarLSQFitter,
+    LinearLSQFitter, LMLSQFitter, NonFiniteValueError, SimplexLSQFitter, SLSQPLSQFitter,
+    TRFLSQFitter, _NLLSQFitter, populate_entry_points)
 from astropy.modeling.optimizers import Optimization
-from astropy.utils import NumpyRNGContext
-from astropy.utils.data import get_pkg_data_filename
 from astropy.stats import sigma_clip
-
+from astropy.utils import NumpyRNGContext
 from astropy.utils.compat.optional_deps import HAS_SCIPY
+from astropy.utils.data import get_pkg_data_filename
 from astropy.utils.exceptions import AstropyUserWarning
-from astropy.modeling.fitting import populate_entry_points
+
 from . import irafutil
 
 if HAS_SCIPY:
@@ -34,12 +34,13 @@ if HAS_SCIPY:
 
 
 fitters = [SimplexLSQFitter, SLSQPLSQFitter]
+non_linear_fitters = [LevMarLSQFitter, TRFLSQFitter, LMLSQFitter, DogBoxLSQFitter]
 
 _RANDOM_SEED = 0x1337
 
 
 class TestPolynomial2D:
-    """Tests for 2D polynomail fitting."""
+    """Tests for 2D polynomial fitting."""
 
     def setup_class(self):
         self.model = models.Polynomial2D(2)
@@ -62,14 +63,30 @@ class TestPolynomial2D:
         assert_allclose(new_model(self.x, self.y), self.z)
 
     @pytest.mark.skipif('not HAS_SCIPY')
-    def test_polynomial2D_nonlinear_fitting(self):
+    @pytest.mark.parametrize('fitter', non_linear_fitters)
+    def test_nonlinear_fitting(self, fitter):
+        fitter = fitter()
+
         self.model.parameters = [.6, 1.8, 2.9, 3.7, 4.9, 6.7]
-        nlfitter = LevMarLSQFitter()
         with pytest.warns(AstropyUserWarning,
                           match=r'Model is linear in parameters'):
-            new_model = nlfitter(self.model, self.x, self.y, self.z)
+            new_model = fitter(self.model, self.x, self.y, self.z)
         assert_allclose(new_model.parameters, [1, 2, 3, 4, 5, 6])
 
+    @pytest.mark.skipif('not HAS_SCIPY')
+    def test_compare_nonlinear_fitting(self):
+        self.model.parameters = [.6, 1.8, 2.9, 3.7, 4.9, 6.7]
+        fit_models = []
+        for fitter in non_linear_fitters:
+            fitter = fitter()
+
+            with pytest.warns(AstropyUserWarning,
+                              match=r'Model is linear in parameters'):
+                fit_models.append(fitter(self.model, self.x, self.y, self.z))
+
+        for pair in combinations(fit_models, 2):
+            assert_allclose(pair[0].parameters, pair[1].parameters)
+
 
 class TestICheb2D:
     """
@@ -101,29 +118,33 @@ class TestICheb2D:
         assert_almost_equal(self.z, z1)
 
     @pytest.mark.skipif('not HAS_SCIPY')
-    def test_chebyshev2D_nonlinear_fitting(self):
+    @pytest.mark.parametrize('fitter', non_linear_fitters)
+    def test_chebyshev2D_nonlinear_fitting(self, fitter):
+        fitter = fitter()
+
         cheb2d = models.Chebyshev2D(2, 2)
         cheb2d.parameters = np.arange(9)
         z = cheb2d(self.x, self.y)
         cheb2d.parameters = [0.1, .6, 1.8, 2.9, 3.7, 4.9, 6.7, 7.5, 8.9]
-        nlfitter = LevMarLSQFitter()
         with pytest.warns(AstropyUserWarning,
                           match=r'Model is linear in parameters'):
-            model = nlfitter(cheb2d, self.x, self.y, z)
+            model = fitter(cheb2d, self.x, self.y, z)
         assert_allclose(model.parameters, [0, 1, 2, 3, 4, 5, 6, 7, 8],
                         atol=10**-9)
 
     @pytest.mark.skipif('not HAS_SCIPY')
-    def test_chebyshev2D_nonlinear_fitting_with_weights(self):
+    @pytest.mark.parametrize('fitter', non_linear_fitters)
+    def test_chebyshev2D_nonlinear_fitting_with_weights(self, fitter):
+        fitter = fitter()
+
         cheb2d = models.Chebyshev2D(2, 2)
         cheb2d.parameters = np.arange(9)
         z = cheb2d(self.x, self.y)
         cheb2d.parameters = [0.1, .6, 1.8, 2.9, 3.7, 4.9, 6.7, 7.5, 8.9]
-        nlfitter = LevMarLSQFitter()
         weights = np.ones_like(self.y)
         with pytest.warns(AstropyUserWarning,
                           match=r'Model is linear in parameters'):
-            model = nlfitter(cheb2d, self.x, self.y, z, weights=weights)
+            model = fitter(cheb2d, self.x, self.y, z, weights=weights)
         assert_allclose(model.parameters, [0, 1, 2, 3, 4, 5, 6, 7, 8],
                         atol=10**-9)
 
@@ -363,40 +384,47 @@ class TestNonLinearFitters:
         self.ydata = func(self.initial_values, self.xdata) + yerror
         self.gauss = models.Gaussian1D(100, 5, stddev=1)
 
-    def test_estimated_vs_analytic_deriv(self):
+    @pytest.mark.parametrize('fitter0', non_linear_fitters)
+    @pytest.mark.parametrize('fitter1', non_linear_fitters)
+    def test_estimated_vs_analytic_deriv(self, fitter0, fitter1):
         """
-        Runs `LevMarLSQFitter` with estimated and analytic derivatives of a
-        `Gaussian1D`.
+        Runs `LevMarLSQFitter` and `TRFLSQFitter` with estimated and
+        analytic derivatives of a `Gaussian1D`.
         """
-
-        fitter = LevMarLSQFitter()
-        model = fitter(self.gauss, self.xdata, self.ydata)
+        fitter0 = fitter0()
+        model = fitter0(self.gauss, self.xdata, self.ydata)
         g1e = models.Gaussian1D(100, 5.0, stddev=1)
-        efitter = LevMarLSQFitter()
-        emodel = efitter(g1e, self.xdata, self.ydata, estimate_jacobian=True)
+
+        fitter1 = fitter1()
+        emodel = fitter1(g1e, self.xdata, self.ydata, estimate_jacobian=True)
         assert_allclose(model.parameters, emodel.parameters, rtol=10 ** (-3))
 
-    def test_estimated_vs_analytic_deriv_with_weights(self):
+    @pytest.mark.parametrize('fitter0', non_linear_fitters)
+    @pytest.mark.parametrize('fitter1', non_linear_fitters)
+    def test_estimated_vs_analytic_deriv_with_weights(self, fitter0, fitter1):
         """
-        Runs `LevMarLSQFitter` with estimated and analytic derivatives of a
-        `Gaussian1D`.
+        Runs `LevMarLSQFitter` and `TRFLSQFitter` with estimated and
+        analytic derivatives of a `Gaussian1D`.
         """
 
         weights = 1.0 / (self.ydata / 10.)
 
-        fitter = LevMarLSQFitter()
-        model = fitter(self.gauss, self.xdata, self.ydata, weights=weights)
+        fitter0 = fitter0()
+        model = fitter0(self.gauss, self.xdata, self.ydata, weights=weights)
         g1e = models.Gaussian1D(100, 5.0, stddev=1)
-        efitter = LevMarLSQFitter()
-        emodel = efitter(g1e, self.xdata, self.ydata, weights=weights, estimate_jacobian=True)
+
+        fitter1 = fitter1()
+        emodel = fitter1(g1e, self.xdata, self.ydata, weights=weights, estimate_jacobian=True)
         assert_allclose(model.parameters, emodel.parameters, rtol=10 ** (-3))
 
-    def test_with_optimize(self):
+    @pytest.mark.parametrize('fitter', non_linear_fitters)
+    def test_with_optimize(self, fitter):
         """
-        Tests results from `LevMarLSQFitter` against `scipy.optimize.leastsq`.
+        Tests results from `LevMarLSQFitter` and `TRFLSQFitter` against
+        `scipy.optimize.leastsq`.
         """
+        fitter = fitter()
 
-        fitter = LevMarLSQFitter()
         model = fitter(self.gauss, self.xdata, self.ydata,
                        estimate_jacobian=True)
 
@@ -410,12 +438,14 @@ class TestNonLinearFitters:
                                   args=(self.xdata, self.ydata))
         assert_allclose(model.parameters, result[0], rtol=10 ** (-3))
 
-    def test_with_weights(self):
+    @pytest.mark.parametrize('fitter', non_linear_fitters)
+    def test_with_weights(self, fitter):
         """
-        Tests results from `LevMarLSQFitter` with weights.
+        Tests results from `LevMarLSQFitter` and `TRFLSQFitter` with weights.
         """
+        fitter = fitter()
+
         # part 1: weights are equal to 1
-        fitter = LevMarLSQFitter()
         model = fitter(self.gauss, self.xdata, self.ydata,
                        estimate_jacobian=True)
         withw = fitter(self.gauss, self.xdata, self.ydata,
@@ -436,45 +466,53 @@ class TestNonLinearFitters:
         assert_allclose(model.parameters, withw.parameters, rtol=10 ** (-4))
 
     @pytest.mark.filterwarnings(r'ignore:.* Maximum number of iterations reached')
-    @pytest.mark.filterwarnings(r'ignore:Values in x were outside bounds during a minimize step, clipping to bounds')
+    @pytest.mark.filterwarnings(r'ignore:Values in x were outside bounds during a minimize step, '
+                                r'clipping to bounds')
     @pytest.mark.parametrize('fitter_class', fitters)
-    def test_fitter_against_LevMar(self, fitter_class):
-        """Tests results from non-linear fitters against `LevMarLSQFitter`."""
+    @pytest.mark.parametrize('fitter', non_linear_fitters)
+    def test_fitter_against_LevMar(self, fitter_class, fitter):
+        """
+        Tests results from non-linear fitters against `LevMarLSQFitter`
+        and `TRFLSQFitter`
+        """
+        fitter = fitter()
 
-        levmar = LevMarLSQFitter()
-        fitter = fitter_class()
+        fitter_cls = fitter_class()
         # This emits a warning from fitter that we need to ignore with
         # pytest.mark.filterwarnings above.
-        new_model = fitter(self.gauss, self.xdata, self.ydata)
-        model = levmar(self.gauss, self.xdata, self.ydata)
+        new_model = fitter_cls(self.gauss, self.xdata, self.ydata)
+        model = fitter(self.gauss, self.xdata, self.ydata)
         assert_allclose(model.parameters, new_model.parameters,
                         rtol=10 ** (-4))
 
-    @pytest.mark.filterwarnings(r'ignore:Values in x were outside bounds during a minimize step, clipping to bounds')
-    def test_LSQ_SLSQP_with_constraints(self):
+    @pytest.mark.filterwarnings(r'ignore:Values in x were outside bounds during a minimize step, '
+                                r'clipping to bounds')
+    @pytest.mark.parametrize('fitter', non_linear_fitters)
+    def test_LSQ_SLSQP_with_constraints(self, fitter):
         """
-        Runs `LevMarLSQFitter` and `SLSQPLSQFitter` on a model with
-        constraints.
+        Runs `LevMarLSQFitter`/`TRFLSQFitter` and `SLSQPLSQFitter` on a
+        model with constraints.
         """
+        fitter = fitter()
 
         g1 = models.Gaussian1D(100, 5, stddev=1)
         g1.mean.fixed = True
-        fitter = LevMarLSQFitter()
         fslsqp = SLSQPLSQFitter()
         slsqp_model = fslsqp(g1, self.xdata, self.ydata)
         model = fitter(g1, self.xdata, self.ydata)
         assert_allclose(model.parameters, slsqp_model.parameters,
                         rtol=10 ** (-4))
 
-    def test_LevMar_with_weights(self):
+    @pytest.mark.parametrize('fitter', non_linear_fitters)
+    def test_non_linear_lsq_fitter_with_weights(self, fitter):
         """
         Tests that issue #11581 has been solved.
         """
+        fitter = fitter()
 
         np.random.seed(42)
         norder = 2
 
-        fitter1 = LevMarLSQFitter()
         fitter2 = LinearLSQFitter()
 
         model = models.Polynomial1D(norder)
@@ -486,7 +524,7 @@ class TestNonLinearFitters:
         ty += np.random.normal(0.0, 1.5, npts)
 
         with pytest.warns(AstropyUserWarning, match=r'Model is linear in parameters'):
-            tf1 = fitter1(model, tx, ty, weights=tw)
+            tf1 = fitter(model, tx, ty, weights=tw)
         tf2 = fitter2(model, tx, ty, weights=tw)
 
         assert_allclose(tf1.parameters, tf2.parameters,
@@ -495,7 +533,11 @@ class TestNonLinearFitters:
                         rtol=10 ** (-2), atol=10 ** (-2))
 
         model = models.Gaussian1D()
-        fitter1(model, tx, ty, weights=tw)
+        if isinstance(fitter, TRFLSQFitter) or isinstance(fitter, LMLSQFitter):
+            with pytest.warns(AstropyUserWarning, match=r'The fit may be unsuccessful; *.'):
+                fitter(model, tx, ty, weights=tw)
+        else:
+            fitter(model, tx, ty, weights=tw)
 
         model = models.Polynomial2D(norder)
         nxpts = 100
@@ -509,7 +551,7 @@ class TestNonLinearFitters:
         tz += np.random.normal(0.0, 1.5, npts).reshape(nxpts, nypts)
 
         with pytest.warns(AstropyUserWarning, match=r'Model is linear in parameters'):
-            tf1 = fitter1(model, tx, ty, tz, weights=tw)
+            tf1 = fitter(model, tx, ty, tz, weights=tw)
         tf2 = fitter2(model, tx, ty, tz, weights=tw)
 
         assert_allclose(tf1.parameters, tf2.parameters,
@@ -539,11 +581,14 @@ class TestNonLinearFitters:
 
         assert_allclose(r_f.parameters, [1.0, 100.0], rtol=1e-2)
 
-    def test_param_cov(self):
+    @pytest.mark.parametrize('fitter', non_linear_fitters)
+    def test_param_cov(self, fitter):
         """
         Tests that the 'param_cov' fit_info entry gets the right answer for
         *linear* least squares, where the answer is exact
         """
+        fitter = fitter()
+
         a = 2
         b = 100
 
@@ -562,7 +607,6 @@ class TestNonLinearFitters:
 
         # now do the non-linear least squares fit
         mod = models.Linear1D(a, b)
-        fitter = LevMarLSQFitter()
 
         with pytest.warns(AstropyUserWarning,
                           match=r'Model is linear in parameters'):
@@ -575,9 +619,6 @@ class TestNonLinearFitters:
 class TestEntryPoint:
     """Tests population of fitting with entry point fitters"""
 
-    def setup_class(self):
-        self.exception_not_thrown = Exception("The test should not have gotten here. There was no exception thrown")
-
     def successfulimport(self):
         # This should work
         class goodclass(Fitter):
@@ -609,54 +650,33 @@ class TestEntryPoint:
 
     def test_import_error(self):
         """This raises an import error on load to test that it is handled correctly"""
-        with warnings.catch_warnings():
-            warnings.filterwarnings('error')
-            try:
-                mock_entry_importerror = mock.create_autospec(EntryPoint)
-                mock_entry_importerror.name = "IErr"
-                mock_entry_importerror.load = self.raiseimporterror
-                populate_entry_points([mock_entry_importerror])
-            except AstropyUserWarning as w:
-                if "ImportError" in w.args[0]:  # any error for this case should have this in it.
-                    pass
-                else:
-                    raise w
-            else:
-                raise self.exception_not_thrown
+
+        mock_entry_importerror = mock.create_autospec(EntryPoint)
+        mock_entry_importerror.name = "IErr"
+        mock_entry_importerror.load = self.raiseimporterror
+
+        with pytest.warns(AstropyUserWarning, match=r".*ImportError.*"):
+            populate_entry_points([mock_entry_importerror])
 
     def test_bad_func(self):
         """This returns a function which fails the type check"""
-        with warnings.catch_warnings():
-            warnings.filterwarnings('error')
-            try:
-                mock_entry_badfunc = mock.create_autospec(EntryPoint)
-                mock_entry_badfunc.name = "BadFunc"
-                mock_entry_badfunc.load = self.returnbadfunc
-                populate_entry_points([mock_entry_badfunc])
-            except AstropyUserWarning as w:
-                if "Class" in w.args[0]:  # any error for this case should have this in it.
-                    pass
-                else:
-                    raise w
-            else:
-                raise self.exception_not_thrown
+
+        mock_entry_badfunc = mock.create_autospec(EntryPoint)
+        mock_entry_badfunc.name = "BadFunc"
+        mock_entry_badfunc.load = self.returnbadfunc
+
+        with pytest.warns(AstropyUserWarning, match=r".*Class.*"):
+            populate_entry_points([mock_entry_badfunc])
 
     def test_bad_class(self):
         """This returns a class which doesn't inherient from fitter """
-        with warnings.catch_warnings():
-            warnings.filterwarnings('error')
-            try:
-                mock_entry_badclass = mock.create_autospec(EntryPoint)
-                mock_entry_badclass.name = "BadClass"
-                mock_entry_badclass.load = self.returnbadclass
-                populate_entry_points([mock_entry_badclass])
-            except AstropyUserWarning as w:
-                if 'modeling.Fitter' in w.args[0]:  # any error for this case should have this in it.
-                    pass
-                else:
-                    raise w
-            else:
-                raise self.exception_not_thrown
+
+        mock_entry_badclass = mock.create_autospec(EntryPoint)
+        mock_entry_badclass.name = "BadClass"
+        mock_entry_badclass.load = self.returnbadclass
+
+        with pytest.warns(AstropyUserWarning, match=r".*BadClass.*"):
+            populate_entry_points([mock_entry_badclass])
 
 
 @pytest.mark.skipif('not HAS_SCIPY')
@@ -671,31 +691,24 @@ class Test1DFittingWithOutlierRemoval:
         self.y = func(self.model_params, self.x)
 
     @pytest.mark.filterwarnings('ignore:The fit may be unsuccessful')
-    @pytest.mark.filterwarnings(r'ignore:Values in x were outside bounds during a minimize step, clipping to bounds')
-    def test_with_fitters_and_sigma_clip(self):
+    @pytest.mark.filterwarnings(r'ignore:Values in x were outside bounds during a minimize step, '
+                                r'clipping to bounds')
+    @pytest.mark.parametrize('fitter', non_linear_fitters + fitters)
+    def test_with_fitters_and_sigma_clip(self, fitter):
         import scipy.stats as stats
 
+        fitter = fitter()
+
         np.random.seed(0)
         c = stats.bernoulli.rvs(0.25, size=self.x.shape)
-        self.y += (np.random.normal(0., 0.2, self.x.shape) +
-                   c*np.random.normal(3.0, 5.0, self.x.shape))
+        y = self.y + (np.random.normal(0., 0.2, self.x.shape) +
+                      c*np.random.normal(3.0, 5.0, self.x.shape))
 
         g_init = models.Gaussian1D(amplitude=1., mean=0, stddev=1.)
-        # test with Levenberg-Marquardt Least Squares fitter
-        fit = FittingWithOutlierRemoval(LevMarLSQFitter(), sigma_clip,
+        fit = FittingWithOutlierRemoval(fitter, sigma_clip,
                                         niter=3, sigma=3.0)
-        fitted_model, _ = fit(g_init, self.x, self.y)
+        fitted_model, _ = fit(g_init, self.x, y)
         assert_allclose(fitted_model.parameters, self.model_params, rtol=1e-1)
-        # test with Sequential Least Squares Programming fitter
-        fit = FittingWithOutlierRemoval(SLSQPLSQFitter(), sigma_clip,
-                                        niter=3, sigma=3.0)
-        fitted_model, _ = fit(g_init, self.x, self.y)
-        assert_allclose(fitted_model.parameters, self.model_params, rtol=1e-1)
-        # test with Simplex LSQ fitter
-        fit = FittingWithOutlierRemoval(SimplexLSQFitter(), sigma_clip,
-                                        niter=3, sigma=3.0)
-        fitted_model, _ = fit(g_init, self.x, self.y)
-        assert_allclose(fitted_model.parameters, self.model_params, atol=1e-1)
 
 
 @pytest.mark.skipif('not HAS_SCIPY')
@@ -733,36 +746,27 @@ class Test2DFittingWithOutlierRemoval:
         return amplitude, x_mean, y_mean
 
     @pytest.mark.filterwarnings('ignore:The fit may be unsuccessful')
-    @pytest.mark.filterwarnings(r'ignore:Values in x were outside bounds during a minimize step, clipping to bounds')
-    def test_with_fitters_and_sigma_clip(self):
+    @pytest.mark.filterwarnings(r'ignore:Values in x were outside bounds during a minimize step, '
+                                r'clipping to bounds')
+    @pytest.mark.parametrize('fitter', non_linear_fitters + fitters)
+    def test_with_fitters_and_sigma_clip(self, fitter):
         import scipy.stats as stats
 
+        fitter = fitter()
+
         np.random.seed(0)
         c = stats.bernoulli.rvs(0.25, size=self.z.shape)
-        self.z += (np.random.normal(0., 0.2, self.z.shape) +
-                   c*np.random.normal(self.z, 2.0, self.z.shape))
+        z = self.z + (np.random.normal(0., 0.2, self.z.shape) +
+                      c*np.random.normal(self.z, 2.0, self.z.shape))
 
         guess = self.initial_guess(self.z, np.array([self.y, self.x]))
         g2_init = models.Gaussian2D(amplitude=guess[0], x_mean=guess[1],
                                     y_mean=guess[2], x_stddev=0.75,
                                     y_stddev=1.25)
 
-        # test with Levenberg-Marquardt Least Squares fitter
-        fit = FittingWithOutlierRemoval(LevMarLSQFitter(), sigma_clip,
+        fit = FittingWithOutlierRemoval(fitter, sigma_clip,
                                         niter=3, sigma=3.)
-        fitted_model, _ = fit(g2_init, self.x, self.y, self.z)
-        assert_allclose(fitted_model.parameters[0:5], self.model_params,
-                        atol=1e-1)
-        # test with Sequential Least Squares Programming fitter
-        fit = FittingWithOutlierRemoval(SLSQPLSQFitter(), sigma_clip, niter=3,
-                                        sigma=3.)
-        fitted_model, _ = fit(g2_init, self.x, self.y, self.z)
-        assert_allclose(fitted_model.parameters[0:5], self.model_params,
-                        atol=1e-1)
-        # test with Simplex LSQ fitter
-        fit = FittingWithOutlierRemoval(SimplexLSQFitter(), sigma_clip,
-                                        niter=3, sigma=3.)
-        fitted_model, _ = fit(g2_init, self.x, self.y, self.z)
+        fitted_model, _ = fit(g2_init, self.x, self.y, z)
         assert_allclose(fitted_model.parameters[0:5], self.model_params,
                         atol=1e-1)
 
@@ -843,13 +847,17 @@ class TestWeightedFittingWithOutlierRemoval:
         assert(fit.parameters[0] > 1.0)     # outliers pulled it high
 
     def test_1d_with_weights_with_sigma_clip(self):
-        """smoke test for #7020 - fails without fitting.py patch because weights does not propagate"""
+        """
+            smoke test for #7020 - fails without fitting.py
+            patch because weights does not propagate
+        """
         model = models.Polynomial1D(0)
         fitter = FittingWithOutlierRemoval(LinearLSQFitter(), sigma_clip,
                                            niter=3, sigma=3.)
         fit, filtered = fitter(model, self.x1d, self.z1d, weights=self.weights1d)
         assert(fit.parameters[0] > 10**(-2))  # weights pulled it > 0
-        assert(fit.parameters[0] < 1.0)       # outliers didn't pull it out of [-1:1] because they had been removed
+        # outliers didn't pull it out of [-1:1] because they had been removed
+        assert(fit.parameters[0] < 1.0)
 
     def test_1d_set_with_common_weights_with_sigma_clip(self):
         """added for #6819 (1D model set with weights in common)"""
@@ -887,9 +895,11 @@ class TestWeightedFittingWithOutlierRemoval:
         assert(mask[0, 0] and mask[0, 1])
         assert_allclose(fit.parameters[0], 0.0, atol=10**(-2))
 
-    def test_2d_with_weights_without_sigma_clip(self):
+    @pytest.mark.parametrize('fitter', non_linear_fitters)
+    def test_2d_with_weights_without_sigma_clip(self, fitter):
+        fitter = fitter()
+
         model = models.Polynomial2D(0)
-        fitter = LevMarLSQFitter()  # LinearLSQFitter doesn't handle weights properly in 2D
         with pytest.warns(AstropyUserWarning,
                           match=r'Model is linear in parameters'):
             fit = fitter(model, self.x, self.y, self.z, weights=self.weights)
@@ -901,35 +911,44 @@ class TestWeightedFittingWithOutlierRemoval:
         fit = fitter(model, self.x, self.y, self.z, weights=self.weights)
         assert(fit.parameters[0] > 1.0)     # outliers pulled it high
 
-    def test_2d_with_weights_with_sigma_clip(self):
+    @pytest.mark.parametrize('base_fitter', non_linear_fitters)
+    def test_2d_with_weights_with_sigma_clip(self, base_fitter):
         """smoke test for #7020 - fails without fitting.py patch because
         weights does not propagate"""
+        base_fitter = base_fitter()
+
         model = models.Polynomial2D(0)
-        fitter = FittingWithOutlierRemoval(LevMarLSQFitter(), sigma_clip,
+        fitter = FittingWithOutlierRemoval(base_fitter, sigma_clip,
                                            niter=3, sigma=3.)
         with pytest.warns(AstropyUserWarning,
                           match=r'Model is linear in parameters'):
-            fit, filtered = fitter(model, self.x, self.y, self.z,
-                                   weights=self.weights)
+            fit, _ = fitter(model, self.x, self.y, self.z, weights=self.weights)
         assert(fit.parameters[0] > 10**(-2))  # weights pulled it > 0
-        assert(fit.parameters[0] < 1.0)       # outliers didn't pull it out of [-1:1] because they had been removed
+        # outliers didn't pull it out of [-1:1] because they had been removed
+        assert(fit.parameters[0] < 1.0)
 
     def test_2d_linear_with_weights_with_sigma_clip(self):
         """same as test above with a linear fitter."""
         model = models.Polynomial2D(0)
         fitter = FittingWithOutlierRemoval(LinearLSQFitter(), sigma_clip,
                                            niter=3, sigma=3.)
-        fit, filtered = fitter(model, self.x, self.y, self.z,
-                               weights=self.weights)
+        fit, _ = fitter(model, self.x, self.y, self.z, weights=self.weights)
         assert(fit.parameters[0] > 10**(-2))  # weights pulled it > 0
-        assert(fit.parameters[0] < 1.0)       # outliers didn't pull it out of [-1:1] because they had been removed
+        # outliers didn't pull it out of [-1:1] because they had been removed
+        assert(fit.parameters[0] < 1.0)
 
 
 @pytest.mark.skipif('not HAS_SCIPY')
-def test_fitters_with_weights():
+@pytest.mark.parametrize('fitter', non_linear_fitters)
+def test_fitters_with_weights(fitter):
     """Issue #5737 """
+    fitter = fitter()
+
+    if isinstance(fitter, _NLLSQFitter):
+        pytest.xfail("This test is poorly designed and causes issues for "
+                     "scipy.optimize.least_squares based fitters")
+
     Xin, Yin = np.mgrid[0:21, 0:21]
-    fitter = LevMarLSQFitter()
 
     with NumpyRNGContext(_RANDOM_SEED):
         zsig = np.random.normal(0, 0.01, size=Xin.shape)
@@ -983,25 +1002,27 @@ def test_linear_fitter_with_weights_flat():
 
 @pytest.mark.skipif('not HAS_SCIPY')
 @pytest.mark.filterwarnings('ignore:The fit may be unsuccessful')
-def test_fitters_interface():
+@pytest.mark.parametrize('fitter', non_linear_fitters + fitters)
+def test_fitters_interface(fitter):
     """
     Test that ``**kwargs`` work with all optimizers.
     This is a basic smoke test.
     """
-    levmar = LevMarLSQFitter()
-    slsqp = SLSQPLSQFitter()
-    simplex = SimplexLSQFitter()
+    fitter = fitter()
 
-    kwargs = {'maxiter': 77, 'verblevel': 1, 'epsilon': 1e-2, 'acc': 1e-6}
-    simplex_kwargs = {'maxiter': 77, 'verblevel': 1, 'acc': 1e-6}
     model = models.Gaussian1D(10, 4, .3)
     x = np.arange(21)
     y = model(x)
 
-    _ = slsqp(model, x, y, **kwargs)
-    _ = simplex(model, x, y, **simplex_kwargs)
-    kwargs.pop('verblevel')
-    _ = levmar(model, x, y, **kwargs)
+    if isinstance(fitter, SimplexLSQFitter):
+        kwargs = {'maxiter': 77, 'verblevel': 1, 'acc': 1e-6}
+    else:
+        kwargs = {'maxiter': 77, 'verblevel': 1, 'epsilon': 1e-2, 'acc': 1e-6}
+
+    if isinstance(fitter, LevMarLSQFitter) or isinstance(fitter, _NLLSQFitter):
+        kwargs.pop('verblevel')
+
+    _ = fitter(model, x, y, **kwargs)
 
 
 @pytest.mark.skipif('not HAS_SCIPY')
@@ -1025,8 +1046,7 @@ def test_optimizers(fitter_class):
     assert fitter._opt_method.acc == 1e-16
 
     # Test repr
-    assert repr(fitter._opt_method) ==\
-        f"{fitter._opt_method.__class__.__name__}()"
+    assert repr(fitter._opt_method) == f"{fitter._opt_method.__class__.__name__}()"
 
     fitparams = mk.MagicMock()
     final_func_val = mk.MagicMock()
@@ -1070,8 +1090,7 @@ def test_Optimization_abstract_call():
     optimization = Optimization(mk.MagicMock())
     with pytest.raises(NotImplementedError) as err:
         optimization()
-    assert str(err.value) ==\
-        "Subclasses should implement this method"
+    assert str(err.value) == "Subclasses should implement this method"
 
 
 def test_fitting_with_outlier_removal_niter():
@@ -1122,6 +1141,7 @@ class TestFittingUncertanties:
                        models.Linear1D(n_models=2, slope=[1., 1.], intercept=[0, 0])]
 
     def setup_class(self):
+        np.random.seed(619)
         self.x = np.arange(10)
         self.x_grid = np.random.randint(0, 100, size=100).reshape(10, 10)
         self.y_grid = np.random.randint(0, 100, size=100).reshape(10, 10)
@@ -1130,12 +1150,14 @@ class TestFittingUncertanties:
 
     @pytest.mark.parametrize(('single_model', 'model_set'),
                              list(zip(example_1D_models, example_1D_sets)))
-    def test_1d_models(self, single_model, model_set):
+    @pytest.mark.parametrize('fitter', non_linear_fitters)
+    def test_1d_models(self, single_model, model_set, fitter):
         """ Test that fitting uncertainties are computed correctly for 1D models
             and 1D model sets. Use covariance/stds given by LevMarLSQFitter as
             a benchmark since they are returned by the numpy fitter.
         """
-        levmar_fitter = LevMarLSQFitter(calc_uncertainties=True)
+        fitter = fitter(calc_uncertainties=True)
+
         linlsq_fitter = LinearLSQFitter(calc_uncertainties=True)
 
         # test 1D single models
@@ -1143,39 +1165,40 @@ class TestFittingUncertanties:
         y = single_model(self.x) + self.rand
         with pytest.warns(AstropyUserWarning,
                           match=r'Model is linear in parameters'):
-            fit_model_levmar = levmar_fitter(single_model, self.x, y)
-        cov_model_levmar = fit_model_levmar.cov_matrix.cov_matrix
+            fit_model = fitter(single_model, self.x, y)
+        cov_model = fit_model.cov_matrix.cov_matrix
 
         # fit single model w/ linlsq fitter
         fit_model_linlsq = linlsq_fitter(single_model, self.x, y)
         cov_model_linlsq = fit_model_linlsq.cov_matrix.cov_matrix
 
         # check covariance, stds computed correctly computed
-        assert_allclose(cov_model_linlsq, cov_model_levmar)
+        assert_allclose(cov_model_linlsq, cov_model)
         assert_allclose(np.sqrt(np.diag(cov_model_linlsq)),
                         fit_model_linlsq.stds.stds)
 
         # now test 1D model sets
         # fit set of models w/ linear fitter
-        y = model_set(self.x, model_set_axis=False) +\
-            np.array([self.rand, self.rand])
+        y = model_set(self.x, model_set_axis=False) + np.array([self.rand, self.rand])
         fit_1d_set_linlsq = linlsq_fitter(model_set, self.x, y)
         cov_1d_set_linlsq = [j.cov_matrix for j in
                              fit_1d_set_linlsq.cov_matrix]
 
         # make sure cov matrix from single model fit w/ levmar fitter matches
         # the cov matrix of first model in the set
-        assert_allclose(cov_1d_set_linlsq[0], cov_model_levmar)
+        assert_allclose(cov_1d_set_linlsq[0], cov_model)
         assert_allclose(np.sqrt(np.diag(cov_1d_set_linlsq[0])),
                         fit_1d_set_linlsq.stds[0].stds)
 
-    def test_2d_models(self):
+    @pytest.mark.parametrize('fitter', non_linear_fitters)
+    def test_2d_models(self, fitter):
         """
         Test that fitting uncertainties are computed correctly for 2D models
         and 2D model sets. Use covariance/stds given by LevMarLSQFitter as
         a benchmark since they are returned by the numpy fitter.
         """
-        levmar_fitter = LevMarLSQFitter(calc_uncertainties=True)
+        fitter = fitter(calc_uncertainties=True)
+
         linlsq_fitter = LinearLSQFitter(calc_uncertainties=True)
         single_model = models.Polynomial2D(2, c0_0=2)
         model_set = models.Polynomial2D(degree=2, n_models=2, c0_0=[2, 3],
@@ -1185,15 +1208,14 @@ class TestFittingUncertanties:
         z_grid = single_model(self.x_grid, self.y_grid) + self.rand_grid
         with pytest.warns(AstropyUserWarning,
                           match=r'Model is linear in parameters'):
-            fit_model_levmar = levmar_fitter(single_model, self.x_grid,
-                                             self.y_grid, z_grid)
-        cov_model_levmar = fit_model_levmar.cov_matrix.cov_matrix
+            fit_model = fitter(single_model, self.x_grid, self.y_grid, z_grid)
+        cov_model = fit_model.cov_matrix.cov_matrix
 
         # fit single model w/ nonlinear fitter
         fit_model_linlsq = linlsq_fitter(single_model, self.x_grid,
                                          self.y_grid, z_grid)
         cov_model_linlsq = fit_model_linlsq.cov_matrix.cov_matrix
-        assert_allclose(cov_model_levmar, cov_model_linlsq)
+        assert_allclose(cov_model, cov_model_linlsq)
         assert_allclose(np.sqrt(np.diag(cov_model_linlsq)),
                         fit_model_linlsq.stds.stds)
 
@@ -1207,7 +1229,7 @@ class TestFittingUncertanties:
 
         # make sure cov matrix from single model fit w/ levmar fitter matches
         # the cov matrix of first model in the set
-        assert_allclose(cov_2d_set_linlsq[0], cov_model_levmar)
+        assert_allclose(cov_2d_set_linlsq[0], cov_model)
         assert_allclose(np.sqrt(np.diag(cov_2d_set_linlsq[0])),
                         fit_2d_set_linlsq.stds[0].stds)
 
@@ -1223,22 +1245,22 @@ class TestFittingUncertanties:
         print(fit_mod.cov_matrix)
         captured = capsys.readouterr()
         assert "slope    | 0.001" in captured.out
-        assert "intercept| -0.006,  0.041" in captured.out
+        assert "intercept| -0.005,  0.03" in captured.out
 
         print(fit_mod.stds)
         captured = capsys.readouterr()
-        assert "slope    | 0.038" in captured.out
-        assert "intercept| 0.203" in captured.out
+        assert "slope    | 0.032" in captured.out
+        assert "intercept| 0.173" in captured.out
 
         # test 'pprint' for Covariance/stds
         print(fit_mod.cov_matrix.pprint(round_val=5, max_lines=1))
         captured = capsys.readouterr()
-        assert "slope    | 0.00144" in captured.out
+        assert "slope    | 0.00105" in captured.out
         assert "intercept" not in captured.out
 
         print(fit_mod.stds.pprint(max_lines=1, round_val=5))
         captured = capsys.readouterr()
-        assert "slope    | 0.03799" in captured.out
+        assert "slope    | 0.03241" in captured.out
         assert "intercept" not in captured.out
 
         # test indexing for Covariance class.
@@ -1246,3 +1268,18 @@ class TestFittingUncertanties:
 
         # test indexing for stds class.
         assert fit_mod.stds[1] == fit_mod.stds['intercept']
+
+
+@pytest.mark.skipif('not HAS_SCIPY')
+def test_non_finite_filter():
+    """Regression test filter introduced to solve issues #3575 and #12809"""
+
+    x = np.array([1, 2, 3, 4, 5, np.nan, 7, np.inf])
+    y = np.array([9, np.nan, 11, np.nan, 13, np.nan, 15, 16])
+
+    m_init = models.Gaussian1D()
+    fit = LevMarLSQFitter()
+
+    # Raise warning, notice fit fails due to nans
+    with pytest.raises(NonFiniteValueError, match=r"Objective function has encountered.*"):
+        fit(m_init, x, y)
