@@ -5,9 +5,9 @@
 """
 import numpy as np
 from numpy import finfo
-from astropy.units.utils import sanitize_scale
-from astropy.units.utils import quantity_asanyarray
+
 from astropy.units.quantity import Quantity
+from astropy.units.utils import quantity_asanyarray, sanitize_scale
 
 _float_finfo = finfo(float)
 
