@@ -13,6 +13,7 @@ from astropy import units as u
 from astropy.units import Quantity
 from astropy.coordinates import Longitude
 from astropy.utils.masked import Masked, MaskedNDArray
+from astropy.utils.compat import NUMPY_LT_1_20
 
 
 def assert_masked_equal(a, b):
@@ -781,6 +782,19 @@ class TestMaskedArrayMethods(MaskedArraySetup):
         assert_array_equal(ma_sum.unmasked, expected_data)
         assert_array_equal(ma_sum.mask, expected_mask)
 
+    @pytest.mark.parametrize('axis', (0, 1, None))
+    def test_sum_where(self, axis):
+        where = np.array([
+            [True, False, False, ],
+            [True, True, True, ],
+        ])
+        where_final = ~self.ma.mask & where
+        ma_sum = self.ma.sum(axis, where=where_final)
+        expected_data = self.ma.unmasked.sum(axis, where=where_final)
+        expected_mask = np.logical_or.reduce(self.ma.mask, axis=axis, where=where_final) | (~where_final).all(axis)
+        assert_array_equal(ma_sum.unmasked, expected_data)
+        assert_array_equal(ma_sum.mask, expected_mask)
+
     @pytest.mark.parametrize('axis', (0, 1, None))
     def test_cumsum(self, axis):
         ma_sum = self.ma.cumsum(axis)
@@ -824,6 +838,22 @@ class TestMaskedArrayMethods(MaskedArraySetup):
         assert result is out
         assert_masked_equal(out, expected)
 
+    @pytest.mark.xfail(NUMPY_LT_1_20, reason="'where' keyword argument not supported for numpy < 1.20")
+    @pytest.mark.filterwarnings("ignore:.*encountered in.*divide")
+    @pytest.mark.filterwarnings("ignore:Mean of empty slice")
+    @pytest.mark.parametrize('axis', (0, 1, None))
+    def test_mean_where(self, axis):
+        where = np.array([
+            [True, False, False, ],
+            [True, True, True, ],
+        ])
+        where_final = ~self.ma.mask & where
+        ma_mean = self.ma.mean(axis, where=where)
+        expected_data = self.ma.unmasked.mean(axis, where=where_final)
+        expected_mask = np.logical_or.reduce(self.ma.mask, axis=axis, where=where_final) | (~where_final).all(axis)
+        assert_array_equal(ma_mean.unmasked, expected_data)
+        assert_array_equal(ma_mean.mask, expected_mask)
+
     @pytest.mark.filterwarnings("ignore:.*encountered in.*divide")
     @pytest.mark.parametrize('axis', (0, 1, None))
     def test_var(self, axis):
@@ -851,6 +881,22 @@ class TestMaskedArrayMethods(MaskedArraySetup):
         expected = ma.astype('f8').var()
         assert_masked_equal(ma_var, expected)
 
+    @pytest.mark.xfail(NUMPY_LT_1_20, reason="'where' keyword argument not supported for numpy < 1.20")
+    @pytest.mark.filterwarnings("ignore:.*encountered in.*divide")
+    @pytest.mark.filterwarnings("ignore:Degrees of freedom <= 0 for slice")
+    @pytest.mark.parametrize('axis', (0, 1, None))
+    def test_var_where(self, axis):
+        where = np.array([
+            [True, False, False, ],
+            [True, True, True, ],
+        ])
+        where_final = ~self.ma.mask & where
+        ma_var = self.ma.var(axis, where=where)
+        expected_data = self.ma.unmasked.var(axis, where=where_final)
+        expected_mask = np.logical_or.reduce(self.ma.mask, axis=axis, where=where_final) | (~where_final).all(axis)
+        assert_array_equal(ma_var.unmasked, expected_data)
+        assert_array_equal(ma_var.mask, expected_mask)
+
     def test_std(self):
         ma_std = self.ma.std(1, ddof=1)
         ma_var1 = self.ma.var(1, ddof=1)
@@ -864,6 +910,22 @@ class TestMaskedArrayMethods(MaskedArraySetup):
         assert result is out
         assert_masked_equal(result, expected)
 
+    @pytest.mark.xfail(NUMPY_LT_1_20, reason="'where' keyword argument not supported for numpy < 1.20")
+    @pytest.mark.filterwarnings("ignore:.*encountered in.*divide")
+    @pytest.mark.filterwarnings("ignore:Degrees of freedom <= 0 for slice")
+    @pytest.mark.parametrize('axis', (0, 1, None))
+    def test_std_where(self, axis):
+        where = np.array([
+            [True, False, False, ],
+            [True, True, True, ],
+        ])
+        where_final = ~self.ma.mask & where
+        ma_std = self.ma.std(axis, where=where)
+        expected_data = self.ma.unmasked.std(axis, where=where_final)
+        expected_mask = np.logical_or.reduce(self.ma.mask, axis=axis, where=where_final) | (~where_final).all(axis)
+        assert_array_equal(ma_std.unmasked, expected_data)
+        assert_array_equal(ma_std.mask, expected_mask)
+
     @pytest.mark.parametrize('axis', (0, 1, None))
     def test_min(self, axis):
         ma_min = self.ma.min(axis)
@@ -879,6 +941,19 @@ class TestMaskedArrayMethods(MaskedArraySetup):
         assert_array_equal(ma_min.unmasked, np.array(2.))
         assert not ma_min.mask
 
+    @pytest.mark.parametrize('axis', (0, 1, None))
+    def test_min_where(self, axis):
+        where = np.array([
+            [True, False, False, ],
+            [True, True, True, ],
+        ])
+        where_final = ~self.ma.mask & where
+        ma_min = self.ma.min(axis, where=where_final, initial=np.inf)
+        expected_data = self.ma.unmasked.min(axis, where=where_final, initial=np.inf)
+        expected_mask = np.logical_or.reduce(self.ma.mask, axis=axis, where=where_final) | (~where_final).all(axis)
+        assert_array_equal(ma_min.unmasked, expected_data)
+        assert_array_equal(ma_min.mask, expected_mask)
+
     @pytest.mark.parametrize('axis', (0, 1, None))
     def test_max(self, axis):
         ma_max = self.ma.max(axis)
@@ -888,6 +963,19 @@ class TestMaskedArrayMethods(MaskedArraySetup):
         assert_array_equal(ma_max.unmasked, expected_data)
         assert not np.any(ma_max.mask)
 
+    @pytest.mark.parametrize('axis', (0, 1, None))
+    def test_max_where(self, axis):
+        where = np.array([
+            [True, False, False, ],
+            [True, True, True, ],
+        ])
+        where_final = ~self.ma.mask & where
+        ma_max = self.ma.max(axis, where=where_final, initial=-np.inf)
+        expected_data = self.ma.unmasked.max(axis, where=where_final, initial=-np.inf)
+        expected_mask = np.logical_or.reduce(self.ma.mask, axis=axis, where=where_final) | (~where_final).all(axis)
+        assert_array_equal(ma_max.unmasked, expected_data)
+        assert_array_equal(ma_max.mask, expected_mask)
+
     @pytest.mark.parametrize('axis', (0, 1, None))
     def test_argmin(self, axis):
         ma_argmin = self.ma.argmin(axis)
@@ -1020,6 +1108,22 @@ class TestMaskedArrayMethods(MaskedArraySetup):
         assert result is out
         assert_masked_equal(result, expected)
 
+    @pytest.mark.xfail(NUMPY_LT_1_20, reason="'where' keyword argument not supported for numpy < 1.20")
+    @pytest.mark.parametrize('method', ('all', 'any'))
+    @pytest.mark.parametrize('axis', (0, 1, None))
+    def test_all_and_any_where(self, method, axis):
+        where = np.array([
+            [True, False, False, ],
+            [True, True, True, ],
+        ])
+        where_final = ~self.ma.mask & where
+        ma_eq = self.ma == self.ma
+        ma_any = getattr(ma_eq, method)(axis, where=where)
+        expected_data = getattr(ma_eq.unmasked, method)(axis, where=where_final)
+        expected_mask = np.logical_or.reduce(self.ma.mask, axis=axis, where=where_final) | (~where_final).all(axis)
+        assert_array_equal(ma_any.unmasked, expected_data)
+        assert_array_equal(ma_any.mask, expected_mask)
+
     @pytest.mark.parametrize('offset', (0, 1))
     def test_diagonal(self, offset):
         mda = self.ma.diagonal(offset=offset)
